package cz.cvut.kbss.bookstore.service;

import cz.cvut.kbss.bookstore.dao.GenericDao;
import java.util.List;
import org.springframework.transaction.annotation.Transactional;

/**
 * Base class for data access services.
 *
 * Defines basic methods for reading, updating, persisting and removing entity
 * instances.
 *
 * @param <T> Entity type
 * @param <ID> Entity id type
 */
public abstract class AbstractDataAccessService<T, ID> implements DataAccessService<T, ID> {

    protected abstract GenericDao<T, ID> getPrimaryDao();

    @Transactional(readOnly = true)
    @Override
    public List<T> findAll() {
        return getPrimaryDao().findAll();
    }

    @Transactional(readOnly = true)
    @Override
    public T find(ID id) {
        return getPrimaryDao().find(id);
    }

    @Transactional
    @Override
    public void persist(T instance) {
        getPrimaryDao().persist(instance);
    }

    @Transactional
    @Override
    public void update(T instance) {
        getPrimaryDao().update(instance);
    }

    @Transactional
    @Override
    public void remove(T instance) {
        getPrimaryDao().remove(instance);
    }

    @Transactional
    @Override
    public void removeById(ID id) {
        final T toRemove = getPrimaryDao().loadById(id);
        if (toRemove != null) {
            getPrimaryDao().remove(toRemove);
        }
    }
}
